// -*- C++ -*-
// $Id: Bessel.CC,v 1.2 2002/05/09 08:02:57 evc Exp $ 
#include "CLHEP/GenericFunctions/Bessel.hh"
#include "CLHEP/GenericFunctions/GSLMetaProxy.h"
#include <math.h>
#include <signal.h>
#include <assert.h>


#define GF_DBL_EPSILON        2.2204460492503131e-16


namespace Genfun {
namespace IntegralOrder {

FUNCTION_OBJECT_IMP(Bessel)

Bessel::Bessel(Type type, unsigned int order):
  _type(type),_order(order)
{
}

Bessel::~Bessel() {
}

Bessel::Bessel(const Bessel & right):
  _type(right._type),
  _order(right._order)
{
}

double Bessel::operator() (double x) const {
  gsl_sf_result result;
  if (_type==J) {
    int status = gsl_sf_bessel_Jn_impl(_order, x,  &result);
    if (status!=GSLMP_NOT_IMPLEMENTED) {
      //
      // Any other return code is a floating point error:
      //
      if (status!=0) {
	std::cerr << "Warning, GSL function gsl_sf_bessel_Jn_impl" 
		  << " return code" << status << std::endl;
	raise(SIGFPE);
      }
      return result.val;
    }
    //
    // And if the GSL implementation fails, there is no other implementation
    // so warn and crash:
    //
    else {
      std::cout << std::endl;
      std::cout << "Error from Genfun::IntegralOrder::Bessel" << std::endl;
      std::cout << "GSL not used and default implementation unavailable, aborting" << std::endl;
      std::cout << std::endl;
      assert(0);
    }
  }
  else if (_type==Y) {
    int status = gsl_sf_bessel_Yn_impl(_order, x,  &result);
    if (status!=GSLMP_NOT_IMPLEMENTED) {
      //
      // Any other return code is a floating point error:
      //
      if (status!=0) {
	std::cerr << "Warning, GSL function gsl_sf_bessel_Yn_impl" 
		  << " return code" << status << std::endl;
	raise(SIGFPE);
      }
      return result.val;
    }
    //
    // And if the GSL implementation fails, there is no other implementation
    // so warn and crash:
    //
    else {
      std::cout << std::endl;
      std::cout << "Error from Genfun::IntegralOrder::Bessel" << std::endl;
      std::cout << "GSL not used and default implementation unavailable, aborting" << std::endl;
      std::cout << std::endl;
      assert(0);
    }
  }
  else {
    return 0;
  }
  return result.val;
}

} // end namespace IntegralOrder

namespace FractionalOrder {

FUNCTION_OBJECT_IMP(Bessel)

Bessel::Bessel(Type type):
  _type(type),
  _order("Order", 0.0,-10,10)
{
}

Bessel::~Bessel() {
}

Bessel::Bessel(const Bessel & right):
  _type(right._type),
  _order(right._order)
{
}


Parameter & Bessel::order() {
  return _order;
}

const Parameter & Bessel::order() const {
  return _order;
}


double Bessel::operator() (double x) const {
  gsl_sf_result result;
  if (_type==J) {
      int status = gsl_sf_bessel_Jnu_impl(_order.getValue(), x,  &result);
      if (status!=GSLMP_NOT_IMPLEMENTED) {
	  //
	  // Any other return code is a floating point error:
	  //
	  if (status!=0) {
	      std::cerr << "Warning, GSL function gsl_sf_bessel_Jnu_impl" 
			<< " return code" << status << std::endl;
	      raise(SIGFPE);
	  }
	  return result.val;
      }
      //
      // And if the GSL implementation fails, there is no other implementation
      // so warn and crash:
      //
      else {
	  std::cout << std::endl;
	  std::cout << "Error from Genfun::FractionaOrder::Bessel" << std::endl;
	  std::cout << "GSL not used and default implementation unavailable, aborting" << std::endl;
	  std::cout << std::endl;
	  assert(0);
      }
  }
  else if (_type==Y) {
      int status = gsl_sf_bessel_Ynu_impl(_order.getValue(), x,  &result);
      if (status!=GSLMP_NOT_IMPLEMENTED) {
	  //
	  // Any other return code is a floating point error:
	  //
	  if (status!=0) {
	      std::cerr << "Warning, GSL function gsl_sf_bessel_Ynu_impl" 
			<< " return code" << status << std::endl;
	      raise(SIGFPE);
	  }
	  return result.val;
      }
      //
      // And if the GSL implementation fails, there is no other implementation
      // so warn and crash:
      //
      else {
	  std::cout << std::endl;
	  std::cout << "Error from Genfun::FractionaOrder::Bessel" << std::endl;
	  std::cout << "GSL not used and default implementation unavailable, aborting" << std::endl;
	  std::cout << std::endl;
	  assert(0);
      }
  }
  return result.val;
}


} // end namespace FractionalOrder

} // end namespace Genfun
