// -*- C++ -*-
// $Id: testBasicVector3D.cc,v 1.2 2001/06/15 08:44:56 evc Exp $
// ---------------------------------------------------------------------------
//
// This file is a part of the CLHEP - a Class Library for High Energy Physics.
//
// This is a test for BasicVector3D class.
//
#include <assert.h>
#include "CLHEP/config/iostream.h"
#include "CLHEP/config/TemplateFunctions.h"
#include "CLHEP/Geometry/Point3D.h"
#include "CLHEP/Geometry/Vector3D.h"
#include "CLHEP/Geometry/Normal3D.h"

typedef HepPoint3D  Point;
typedef HepVector3D Vector;
typedef HepNormal3D Normal;

#define DEL 10.e-16

int main() {
  // Constructors
  Point  p;
  assert ( p.x() == 0.0 );
  assert ( p.y() == 0.0 );
  assert ( p.z() == 0.0 );

  Vector v(1,2,3);
  assert ( v.x() == 1.0 );
  assert ( v.y() == 2.0 );
  assert ( v.z() == 3.0 );

  Normal n(-v);
  assert ( n.x() == -1.0 );
  assert ( n.y() == -2.0 );
  assert ( n.z() == -3.0 );

  // Modify/assignment
  p  = v;
  n *= -1;
  assert ( p == n);

  v += +p;
  n -= -p;
  assert ( v == n);

  // Indexing
  p.set(3,2,1);
  v.setX(4);
  v.setY(3);
  v.setZ(2);

  assert ( v[v.X] == 4.0 );
  assert ( v[v.Y] == 3.0 ) ;
  assert ( v[v.Z] == 2.0 );
 
  assert (Point::NUM_COORDINATES == 3);

  for (int i=0; i<p.NUM_COORDINATES; i++) { assert ( p(i) == (v[i] - 1.) ); }

  // Dot/Cross products
  assert ( p*v == 20. ); 
  n = p.cross(v);
  assert ( n == Vector(1,-2,1)); 

  // Arithmetics
  assert ( p+v == Point(7,5,3)     );
  assert ( v-n == Normal(3,5,1)    );
  assert (-2*p == Vector(-6,-4,-2) );
  assert ( v*3 == Point(12,9,6)    );

  // Input/Output
  HepStd::cin  >> p;
  HepStd::cout << p << HepStd::endl;

  return 0;
}
